@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,8 +50,9 @@
 package com.openexchange.data.conversion.ical.ical4j;
 
 import net.fortuna.ical4j.model.Calendar;
-
 import com.openexchange.data.conversion.ical.ICalSession;
+import com.openexchange.data.conversion.ical.Mode;
+import com.openexchange.data.conversion.ical.ZoneInfo;
 
 /**
  *
@@ -59,14 +60,25 @@ import com.openexchange.data.conversion.ical.ICalSession;
  */
 public final class ICal4jSession implements ICalSession {
 
-    private final Calendar calendar = new Calendar();
+    private Calendar calendar = new Calendar();
+    private final Mode mode;
     private int index;
 
     /**
      * Default constructor.
+     * @param mode
      */
-    public ICal4jSession() {
+    public ICal4jSession(Mode mode) {
         super();
+        this.mode = mode;
+    }
+
+    public Mode getMode() {
+        return mode;
+    }
+
+    public ZoneInfo getZoneInfo() {
+        return mode.getZoneInfo();
     }
 
     /**
@@ -76,6 +88,11 @@ public final class ICal4jSession implements ICalSession {
         return calendar;
     }
 
+    public void setCalendar(Calendar cal) {
+        calendar = cal;
+        index = 0;
+    }
+    
     /**
      * Counts the number of elements already parsed for error messages
      * @return
@@ -83,6 +100,6 @@ public final class ICal4jSession implements ICalSession {
     public int getAndIncreaseIndex() {
         return index++;
     }
-    
-    
+
+
 }
