@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,14 +51,20 @@ package com.openexchange.data.conversion.ical.ical4j.internal;
 
 import java.util.TimeZone;
 import net.fortuna.ical4j.model.DateTime;
+import net.fortuna.ical4j.model.TimeZoneRegistry;
+import net.fortuna.ical4j.model.TimeZoneRegistryFactory;
 import net.fortuna.ical4j.util.TimeZones;
+import net.fortuna.ical4j.zoneinfo.outlook.OutlookTimeZoneRegistryFactory;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import com.openexchange.api2.OXException;
+import com.openexchange.data.conversion.ical.ZoneInfo;
 import com.openexchange.groupware.calendar.CalendarCollectionService;
 import com.openexchange.groupware.calendar.CalendarDataObject;
 import com.openexchange.groupware.calendar.Constants;
 import com.openexchange.groupware.calendar.RecurringResultsInterface;
+import com.openexchange.groupware.container.Appointment;
+import com.openexchange.groupware.container.CalendarObject;
 
 /**
  *
@@ -66,22 +72,54 @@ import com.openexchange.groupware.calendar.RecurringResultsInterface;
  */
 public final class EmitterTools {
 
-    private static final Log LOG = LogFactory.getLog(EmitterTools.class);
-
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(EmitterTools.class));
     private static CalendarCollectionService calendarCollection;
 
-    /**
-     * Prevent instantiation.
-     */
-    private EmitterTools() {
+    private final TimeZoneRegistry registry;
+
+    public EmitterTools(ZoneInfo zoneInfo) {
         super();
+        registry = getRegistry(zoneInfo);
+    }
+
+    private static final TimeZoneRegistry getRegistry(final ZoneInfo zoneInfo) {
+        final TimeZoneRegistry retval;
+        switch (zoneInfo) {
+        case OUTLOOK:
+            retval = new OutlookTimeZoneRegistryFactory().createRegistry();
+            break;
+        case FULL:
+        default:
+            retval = TimeZoneRegistryFactory.getInstance().createRegistry();
+            break;
+        }
+        return retval;
     }
 
-    /**
-     * TODO add default timezone
-     */
     public static DateTime toDateTime(final java.util.Date date) {
-        final DateTime retval = new DateTime(true);
+    	final DateTime retval = new DateTime(true);
+        retval.setTime(date.getTime());
+        return retval;
+    }
+
+    public DateTime toDateTime(final java.util.Date date, final String tzid) {
+        return toDateTime(registry, date, tzid);
+    }
+
+    public static DateTime toDateTime(final ZoneInfo zoneInfo, final java.util.Date date, final String tzid) {
+        return toDateTime(getRegistry(zoneInfo), date, tzid);
+    }
+
+    private static DateTime toDateTime(final TimeZoneRegistry registry, final java.util.Date date, final String tzid) {
+        if (null == tzid) {
+            return toDateTime(date);
+        }
+        net.fortuna.ical4j.model.TimeZone ical4jTimezone = registry.getTimeZone(tzid);
+        if (null == ical4jTimezone) {
+            return toDateTime(date);
+        }
+        final DateTime retval = new DateTime(false);
+        retval.setTimeZone(ical4jTimezone);
         retval.setTime(date.getTime());
         return retval;
     }
@@ -90,6 +128,18 @@ public final class EmitterTools {
         return new UTCDate(date.getTime());
     }
 
+    public static net.fortuna.ical4j.model.Date toDate(final java.util.Date date, String tzid) {
+    	if(tzid == null)
+    		return toDate(date);
+        return new TzDate(date.getTime(), tzid);
+    }
+
+    public static String extractTimezoneIfPossible(CalendarObject co){
+    	if(Appointment.class.isAssignableFrom(co.getClass()))
+    		return ((Appointment) co).getTimezone();
+    	return null;
+    }
+
     /**
      * {@link Date} normally uses the JVM default time zone. This shifts whole
      * day appointments one day earlier for {@link TimeZone}s that have a
@@ -107,6 +157,17 @@ public final class EmitterTools {
         }
     }
 
+    private static final class TzDate extends net.fortuna.ical4j.model.Date {
+
+        private static final long serialVersionUID = -4317836084736029666L;
+
+        public TzDate(final long time, final String tzid) {
+            super();
+            getFormat().setTimeZone(TimeZone.getTimeZone(tzid));
+            setTime(time);
+        }
+    }
+
     public static java.util.Date calculateExactTime(final CalendarDataObject appointment, final java.util.Date exception) {
         java.util.Date retval = exception;
         try {
@@ -127,7 +188,11 @@ public final class EmitterTools {
         return retval;
     }
 
-    public static void setCalendarCollection(CalendarCollectionService calendarCollection) {
+    public static void setCalendarCollection(final CalendarCollectionService calendarCollection) {
         EmitterTools.calendarCollection = calendarCollection;
     }
+
+    public TimeZoneRegistry getTimeZoneRegistry() {
+        return registry;
+    }
 }
