@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,15 +51,16 @@ package com.openexchange.data.conversion.ical.ical4j.internal;
 
 import java.util.Date;
 import java.util.GregorianCalendar;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.TimeZone;
 import net.fortuna.ical4j.model.DateTime;
 import net.fortuna.ical4j.model.Property;
 import net.fortuna.ical4j.model.component.CalendarComponent;
 import net.fortuna.ical4j.model.property.DateProperty;
+import org.apache.commons.collections.BidiMap;
+import org.apache.commons.collections.bidimap.TreeBidiMap;
 
-/**
- * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
- */
 public final class ParserTools {
 
     /**
@@ -77,23 +78,16 @@ public final class ParserTools {
         }
         return retval;
     }
-    
+
     /**
      * Parses a date. If the value is a datetime, the timezone will be applied if needed, if the value is a date
      * the time will be 00:00 UTC
      */
     public static Date parseDateConsideringDateType(final CalendarComponent component, final DateProperty property, final TimeZone timeZone) {
-        final boolean isDateTime = isDateTime(component, property);
-        final TimeZone UTC = TimeZone.getTimeZone("UTC");
-        final Date value;
-        if (isDateTime) {
-            value = parseDate(component, property, timeZone);
-        } else {
-            value = parseDate(component, property, UTC);
-        }
-        return value;
+        DateProperty value = (DateProperty)component.getProperty(property.getName());
+        return toDateConsideringDateType(value, timeZone);
     }
-    
+
     /**
      * Parses a date. If the value is a datetime, the timezone will be applied if needed, if the value is a date
      * the time will be 00:00 UTC
@@ -103,7 +97,10 @@ public final class ParserTools {
         final TimeZone UTC = TimeZone.getTimeZone("UTC");
         Date date;
         if (isDateTime) {
-            date = toDate(value, timeZone);
+            date = new Date(value.getDate().getTime());
+            if (inDefaultTimeZone(value, timeZone)) {
+                date = recalculate(date, timeZone);
+            }
         } else {
             date = toDate(value, UTC);
         }
@@ -114,12 +111,12 @@ public final class ParserTools {
         return isDateTime(component, property.getName());
     }
 
-    public static boolean isDateTime(final CalendarComponent component, String name) {
+    public static boolean isDateTime(final CalendarComponent component, final String name) {
         final DateProperty value = (DateProperty) component.getProperty(name);
         return isDateTime(value);
     }
 
-    public static boolean isDateTime(DateProperty value) {
+    public static boolean isDateTime(final DateProperty value) {
         return value.getDate() instanceof DateTime;
     }
 
@@ -151,12 +148,27 @@ public final class ParserTools {
         return inTimeZone.getTime();
     }
 
+    /**
+     * Transforms date from the default timezone to midnight in the given timezone.
+     */
+    public static Date recalculateMidnight(final Date date, final TimeZone timeZone) {
+
+        final java.util.Calendar inDefault = new GregorianCalendar();
+        inDefault.setTime(date);
+
+        final java.util.Calendar inTimeZone = new GregorianCalendar();
+        inTimeZone.setTimeZone(timeZone);
+        inTimeZone.set(
+            inDefault.get(java.util.Calendar.YEAR),
+            inDefault.get(java.util.Calendar.MONTH),
+            inDefault.get(java.util.Calendar.DATE),
+            0,0,0);
+        inTimeZone.set(java.util.Calendar.MILLISECOND, 0);
+        return inTimeZone.getTime();
+    }
+
     public static Date toDate(final DateProperty dateProperty, final TimeZone tz) {
-        Date date = new Date(dateProperty.getDate().getTime());
-        if (ParserTools.inDefaultTimeZone(dateProperty, tz)) {
-            date = ParserTools.recalculate(date, tz);
-        }
-        return date;
+        return new Date(dateProperty.getDate().getTime());
     }
 
     public static Date recalculateAsNeeded(final net.fortuna.ical4j.model.Date icaldate, final Property property, final TimeZone tz) {
@@ -172,4 +184,80 @@ public final class ParserTools {
         }
         return new Date(icaldate.getTime());
     }
+    
+    /**
+     * Takes an Outlook timezone id and guesses the appropriate 
+     * Java timezone. It does so by counting all timezones that
+     * have somewhat matching names, sorts them by their UTC 
+     * offset and chooses one of those that have the most often
+     * used offset, e.g. if you have three zones that have a +1 
+     * offset and one with +2, you'll get one of the +1 ones.
+     *  
+     * This thing would be way better if we had the timezone 
+     * information that we probably painfully extracted somewhere 
+     * in ical4j but got rid off again before this method is 
+     * needed. 
+     */
+    public static TimeZone findTzidBySimilarity(final String tzidName) {
+    	if("Z".equals(tzidName)){
+    		return TimeZone.getTimeZone("Zulu");
+    	}
+    	//generate name variations of the outlook timezone name
+    	final List<String> candidates1 = new LinkedList<String>();
+    	if(tzidName.contains(",")){
+    		final String[] split = tzidName.split(",");
+    		for(final String tmp: split) {
+                candidates1.add(tmp.trim());
+            }
+    	} else {
+    		candidates1.add(tzidName);
+    	}
+    	//compare all Java timezones to the candidates
+    	final String[] availableIDs = TimeZone.getAvailableIDs();
+    	final List<TimeZone> candidates2 = new LinkedList<TimeZone>();
+    	for(final String javaId: availableIDs){
+    		for(final String idPart: candidates1){
+    			if(javaId.equals(idPart)){
+    				candidates2.add(TimeZone.getTimeZone(javaId));
+    			}
+    			if(javaId.equalsIgnoreCase(idPart)){
+    				candidates2.add(TimeZone.getTimeZone(javaId));
+    			}
+    			if(javaId.contains(idPart)){
+    				candidates2.add(TimeZone.getTimeZone(javaId));
+    			}
+    			if(javaId.toLowerCase().contains(idPart.toLowerCase())){
+    				candidates2.add(TimeZone.getTimeZone(javaId));
+    			}
+    		}
+    	}
+		//now count how many different offsets there are
+    	final BidiMap occurrences = new TreeBidiMap(); 
+    	int highestNumberOccurrences = 0;
+    	for(final TimeZone cand: candidates2){
+    		final int offset = cand.getRawOffset();
+    		if(!occurrences.containsKey(offset)) {
+                occurrences.put(offset, 0);
+            }
+    		final int numOccurrences = ((Integer)occurrences.get(offset))+1;
+    		occurrences.put(offset, numOccurrences);
+    		highestNumberOccurrences = highestNumberOccurrences < numOccurrences ? numOccurrences : highestNumberOccurrences; 
+    	}
+    	//select the most often occurring ones and take the one with the shortest name (probably a generic name)
+    	final Integer mostCommonOffset = (Integer) occurrences.getKey(highestNumberOccurrences);
+    	int maxlength = Integer.MAX_VALUE;
+    	TimeZone candidate = null;
+    	for(final TimeZone cand: candidates2){
+    		if(cand.getRawOffset() == mostCommonOffset){
+    			final int l2 = cand.getID().length();
+    			if(l2 < maxlength){
+    				candidate = cand;
+    				maxlength = l2; 
+    			}
+    		}
+    	}
+    	return candidate;
+	}
+    
+    
 }
