@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,19 +49,21 @@
 
 package com.openexchange.data.conversion.ical.ical4j.internal.appointment;
 
+import java.util.Calendar;
 import java.util.List;
 import java.util.TimeZone;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import net.fortuna.ical4j.model.component.VEvent;
 import net.fortuna.ical4j.model.property.RecurrenceId;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import com.openexchange.api2.OXException;
-import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools;
+import com.openexchange.data.conversion.ical.ical4j.internal.ParserTools;
 import com.openexchange.groupware.calendar.CalendarCollectionService;
 import com.openexchange.groupware.calendar.CalendarDataObject;
 import com.openexchange.groupware.calendar.OXCalendarException;
@@ -75,7 +77,8 @@ import com.openexchange.groupware.contexts.Context;
  */
 public class ChangeExceptions extends AbstractVerifyingAttributeConverter<VEvent, Appointment> {
 
-    private static final Log LOG = LogFactory.getLog(ChangeExceptions.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(ChangeExceptions.class));
+    private static final TimeZone UTC = TimeZone.getTimeZone("UTC");
 
     private static CalendarCollectionService calendarCollection;
 
@@ -87,7 +90,7 @@ public class ChangeExceptions extends AbstractVerifyingAttributeConverter<VEvent
         return appointment.isException();
     }
 
-    public void emit(final int index, final Appointment appointment, final VEvent vEvent, final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
+    public void emit(final Mode mode, final int index, final Appointment appointment, final VEvent vEvent, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
         final java.util.Date changeException = appointment.getRecurrenceDatePosition();
         if (null == changeException) {
             return;
@@ -97,8 +100,13 @@ public class ChangeExceptions extends AbstractVerifyingAttributeConverter<VEvent
             final CalendarDataObject cloned = (CalendarDataObject) appointment.clone();
             try {
                 calendarCollection.fillDAO(cloned);
-                cloned.setStartDate(parseSeriesStart(cloned.getRecurrence()));
-                date = EmitterTools.toDateTime(EmitterTools.calculateExactTime(cloned, changeException));
+                String recurrence = cloned.getRecurrence();
+                if (recurrence == null) {
+                	date = EmitterTools.toDate(changeException);
+                } else {
+    				cloned.setStartDate(parseSeriesStart(recurrence));
+                    date = EmitterTools.toDateTime(EmitterTools.calculateExactTime(cloned, changeException));
+                }
             } catch (final OXException e) {
                 LOG.warn(e.getMessage(), e);
                 date = EmitterTools.toDate(changeException);
@@ -113,6 +121,9 @@ public class ChangeExceptions extends AbstractVerifyingAttributeConverter<VEvent
     private static final Pattern startDatePattern = Pattern.compile("s\\|(\\d*)\\|");
 
     private java.util.Date parseSeriesStart(final String recurrenceString) throws OXException {
+    	if (recurrenceString == null)  {
+    		return null;
+    	}
         final Matcher matcher = startDatePattern.matcher(recurrenceString);
         final long start;
         if (matcher.find()) {
@@ -128,17 +139,30 @@ public class ChangeExceptions extends AbstractVerifyingAttributeConverter<VEvent
     }
 
     public boolean hasProperty(final VEvent vEvent) {
-        return false;
+        return vEvent.getRecurrenceId() != null;
     }
 
-    public void parse(final int index, final VEvent vEvent, final Appointment appointment, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) throws ConversionError {
-        // Currently unimplemented. Series must be changed for change exceptions.
+    public void parse(final int index, final VEvent vEvent, final Appointment appointment, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
+        RecurrenceId property = vEvent.getRecurrenceId();
+        java.util.Date date = ParserTools.recalculateAsNeeded(property.getDate(), property, timeZone);
+        java.util.Date recurrencePosition = normalize(date);
+        appointment.setRecurrenceDatePosition(recurrencePosition);
+    }
+
+    private java.util.Date normalize(java.util.Date date) {
+        Calendar cal = Calendar.getInstance(UTC);
+        cal.setTime(date);
+        cal.set(Calendar.HOUR_OF_DAY, 0);
+        cal.set(Calendar.MINUTE, 0);
+        cal.set(Calendar.SECOND, 0);
+        cal.set(Calendar.MILLISECOND, 0);
+        return cal.getTime();
     }
 
     /**
      * TODO Refactor this to add {@link ChangeExceptions} dynamically depending on service in AppointmentConverters.
      */
-    public static void setCalendarCollection(CalendarCollectionService calendarCollection) {
+    public static void setCalendarCollection(final CalendarCollectionService calendarCollection) {
         ChangeExceptions.calendarCollection = calendarCollection;
     }
 }
