@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,14 +49,16 @@
 
 package com.openexchange.data.conversion.ical.ical4j.internal.appointment;
 
+import java.util.Calendar;
+import java.util.Date;
 import java.util.List;
 import java.util.TimeZone;
 import net.fortuna.ical4j.model.DateList;
 import net.fortuna.ical4j.model.PropertyList;
 import net.fortuna.ical4j.model.component.VEvent;
 import net.fortuna.ical4j.model.property.ExDate;
-import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools;
 import com.openexchange.data.conversion.ical.ical4j.internal.ParserTools;
@@ -75,20 +77,20 @@ public class DeleteExceptions extends AbstractVerifyingAttributeConverter<VEvent
     public DeleteExceptions() {
         super();
     }
-    
+
     public boolean isSet(final Appointment appointment) {
         return appointment.isMaster() && appointment.containsDeleteExceptions();
     }
 
-    public void emit(final int index, final Appointment appointment, final VEvent vEvent, final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
+    public void emit(final Mode mode, final int index, final Appointment appointment, final VEvent vEvent, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
         final java.util.Date[] dates = appointment.getDeleteException();
         if (null == dates) {
             return;
         }
         // Only when the DateList is created this way the correct dates are written to iCal file.
-        final DateList deleteExceptions = new DateList(dates.length);
-        deleteExceptions.setUtc(true);
         for (final java.util.Date deleteException : dates) {
+            final DateList deleteExceptions = new DateList();
+            deleteExceptions.setUtc(true);
             final net.fortuna.ical4j.model.Date date;
             if (CalendarDataObject.class.isAssignableFrom(appointment.getClass())) {
                 final CalendarDataObject cloned = (CalendarDataObject) appointment.clone();
@@ -97,28 +99,37 @@ public class DeleteExceptions extends AbstractVerifyingAttributeConverter<VEvent
                 date = EmitterTools.toDateTime(deleteException);
             }
             deleteExceptions.add(date);
+            vEvent.getProperties().add(new ExDate(deleteExceptions));
         }
-        final ExDate exDates = new ExDate(deleteExceptions);
-        vEvent.getProperties().add(exDates);
     }
 
     public boolean hasProperty(final VEvent vEvent) {
         return null != vEvent.getProperty("EXDATE");
     }
 
-    public void parse(final int index, final VEvent vEvent, final Appointment appointment, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) throws ConversionError {
+    public void parse(final int index, final VEvent vEvent, final Appointment appointment, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
         final PropertyList exdates = vEvent.getProperties("EXDATE");
         final int size = exdates.size();
         for (int i = 0; i < size; i++) {
-            final ExDate exdate = (ExDate) exdates.get(0);
+            final ExDate exdate = (ExDate) exdates.get(i);
 
             final DateList dates = exdate.getDates();
             final int size2 = dates.size();
             for (int j = 0; j < size2; j++) {
                 final net.fortuna.ical4j.model.Date icaldate = (net.fortuna.ical4j.model.Date) dates.get(j);
                 final java.util.Date date = ParserTools.recalculateAsNeeded(icaldate, exdate, timeZone);
-                appointment.addDeleteException(date);
+                appointment.addDeleteException(normalize(date));
             }
         }
     }
+
+	private Date normalize(Date date) {
+		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+		cal.setTime(date);
+		cal.set(Calendar.HOUR_OF_DAY, 0);
+		cal.set(Calendar.MINUTE, 0);
+		cal.set(Calendar.SECOND, 0);
+		cal.set(Calendar.MILLISECOND, 0);
+		return cal.getTime();
+	}
 }
