@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,10 +51,9 @@ package com.openexchange.data.conversion.ical.ical4j.internal.appointment;
 
 import java.util.List;
 import java.util.TimeZone;
-
 import net.fortuna.ical4j.model.component.VEvent;
-
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.groupware.contexts.Context;
@@ -65,44 +64,26 @@ import com.openexchange.groupware.contexts.Context;
  */
 public final class IgnoreConflicts extends AbstractVerifyingAttributeConverter<VEvent, Appointment> {
 
-    /**
-     * Default constructor.
-     */
     public IgnoreConflicts() {
         super();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean isSet(final Appointment appointment) {
         // Ignore this always on output.
         return false;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void emit(final int index, final Appointment appointment, final VEvent vEvent,
-        final List<ConversionWarning> warnings, final Context ctx, Object... args) {
+    public void emit(final Mode mode, final int index, final Appointment appointment, final VEvent vEvent, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
         // Ignore this always on output.
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean hasProperty(final VEvent vEvent) {
         // Set it always on input.
         return true;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void parse(final int index, final VEvent vEvent, final Appointment appointment,
-        final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
+    public void parse(final int index, final VEvent vEvent, final Appointment appointment, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
         // Set it always on input.
         appointment.setIgnoreConflicts(true);
     }
-
 }
