@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -61,30 +61,37 @@ import java.util.List;
 import java.util.Map;
 import java.util.TimeZone;
 import net.fortuna.ical4j.model.NumberList;
+import net.fortuna.ical4j.model.Property;
 import net.fortuna.ical4j.model.PropertyList;
 import net.fortuna.ical4j.model.Recur;
 import net.fortuna.ical4j.model.WeekDay;
 import net.fortuna.ical4j.model.WeekDayList;
 import net.fortuna.ical4j.model.component.CalendarComponent;
+import net.fortuna.ical4j.model.property.DtStart;
 import net.fortuna.ical4j.model.property.RRule;
 import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
 import com.openexchange.data.conversion.ical.ConversionWarning.Code;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.data.conversion.ical.ical4j.internal.EmitterTools;
 import com.openexchange.data.conversion.ical.ical4j.internal.ParserTools;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.groupware.container.CalendarObject;
 import com.openexchange.groupware.contexts.Context;
+import com.openexchange.groupware.tasks.Task;
 
 /**
  * @author Francisco Laguna <francisco.laguna@open-xchange.com>
  */
-public class Recurrence<T extends CalendarComponent, U extends CalendarObject> extends AbstractVerifyingAttributeConverter<T,U> {
+public class Recurrence<T extends CalendarComponent, U extends CalendarObject> extends AbstractVerifyingAttributeConverter<T, U> {
 
     private static final Map<String, Integer> weekdays = new HashMap<String, Integer>();
+
     private static final Map<Integer, String> reverseDays = new HashMap<Integer, String>();
+
     private static final List<Integer> allDays = new LinkedList<Integer>();
+
     private static final SimpleDateFormat date;
     static {
         weekdays.put("MO", Integer.valueOf(Appointment.MONDAY));
@@ -95,7 +102,7 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
         weekdays.put("SA", Integer.valueOf(Appointment.SATURDAY));
         weekdays.put("SU", Integer.valueOf(Appointment.SUNDAY));
 
-        for(final Map.Entry<String, Integer> entry : weekdays.entrySet()) {
+        for (final Map.Entry<String, Integer> entry : weekdays.entrySet()) {
             allDays.add(entry.getValue());
             reverseDays.put(entry.getValue(), entry.getKey());
         }
@@ -115,28 +122,28 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
      * {@inheritDoc}
      */
     public boolean isSet(final U calendar) {
-        return calendar.containsRecurrenceType();
+        return calendar.containsRecurrenceType() && calendar.getRecurrenceDatePosition() == null && calendar.getRecurrencePosition() == 0;
     }
 
-    public void emit(final int index, final U calendar, final T component, final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
-        if(calendar.isException()) {
+    public void emit(final Mode mode, final int index, final U calendar, final T component, final List<ConversionWarning> warnings, final Context ctx, final Object... args) throws ConversionError {
+        if (calendar.isException()) {
             return;
         }
         switch (calendar.getRecurrenceType()) {
-            case CalendarObject.DAILY:
-                addDailyRecurrence(calendar, component);
-                break;
-            case CalendarObject.WEEKLY:
-                addWeeklyRecurrence(index, calendar, component);
-                break;
-            case CalendarObject.MONTHLY:
-                addMonthlyRecurrence(index, calendar, component);
-                break;
-            case CalendarObject.YEARLY:
-                addYearlyRecurrence(index, calendar, component);
-                break;
-            default:
-                return;
+        case CalendarObject.DAILY:
+            addDailyRecurrence(calendar, component);
+            break;
+        case CalendarObject.WEEKLY:
+            addWeeklyRecurrence(index, calendar, component);
+            break;
+        case CalendarObject.MONTHLY:
+            addMonthlyRecurrence(index, calendar, component);
+            break;
+        case CalendarObject.YEARLY:
+            addYearlyRecurrence(index, calendar, component);
+            break;
+        default:
+            return;
         }
     }
 
@@ -181,7 +188,7 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
             final RRule rrule = new RRule(new Recur(recur.toString()));
             component.getProperties().add(rrule);
         } catch (final ParseException e) {
-            throw new ConversionError(index,ConversionError.Code.CANT_CREATE_RRULE, e, recur.toString());
+            throw new ConversionError(index, ConversionError.Code.CANT_CREATE_RRULE, e, recur.toString());
         }
     }
 
@@ -200,9 +207,7 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
         if (calendar.containsOccurrence()) {
             recur.append(";COUNT=").append(calendar.getOccurrence());
         } else if (calendar.containsUntil()) {
-            synchronized (date) {
-                recur.append(";UNTIL=").append(date.format(calendar.getUntil()));
-            }
+            recur.append(";UNTIL=").append(getUntil(calendar).toString());
         }
         return recur;
     }
@@ -218,8 +223,10 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
         final Recur retval;
         if (calendar.containsOccurrence()) {
             retval = new Recur(frequency, calendar.getOccurrence());
+        } else if (calendar.containsUntil()) {
+            retval = new Recur(frequency, getUntil(calendar));
         } else {
-            retval = new Recur(frequency, EmitterTools.toDate(calendar.getUntil()));
+            retval = new Recur(frequency, null);
         }
         return retval;
     }
@@ -243,7 +250,14 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
         }
         final Recur rrule = ((RRule) list.get(0)).getRecur();
         if ("DAILY".equalsIgnoreCase(rrule.getFrequency())) {
-            cObj.setRecurrenceType(Appointment.DAILY);
+            if (null != rrule.getDayList() && 0 < rrule.getDayList().size()) {
+                // used as "each weekday" by some clients: FREQ=DAILY;INTERVAL=1;WKST=SU;BYDAY=MO,TU,WE,TH,FR
+                // save as 'weekly' type with daymask
+                cObj.setRecurrenceType(Appointment.WEEKLY);
+                setDays(index, cObj, rrule, startDate);
+            } else {
+                cObj.setRecurrenceType(Appointment.DAILY);
+            }
             if (!rrule.getMonthList().isEmpty()) {
                 throw new ConversionError(index, Code.BYMONTH_NOT_SUPPORTED);
             }
@@ -277,10 +291,11 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
             cObj.setRecurrenceCount(recurrenceCount);
             setOccurrenceIfNeededRecoveryFIXME(cObj, recurrenceCount);
         } else if (null != rrule.getUntil()) {
-            cObj.setUntil(ParserTools.recalculate(new Date(rrule.getUntil().getTime()), timeZone));
+            cObj.setUntil(getUntil(rrule, component));
         }
     }
 
+
     private void setOccurrenceIfNeededRecoveryFIXME(final U cObj, final int recurrenceCount) {
         if (Appointment.class.isAssignableFrom(cObj.getClass())) {
             cObj.setOccurrence(recurrenceCount);
@@ -310,10 +325,10 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
         }
     }
 
-    private void setDayInMonthFromSetPos(int index, CalendarObject obj, Recur rrule) {
-        if(!rrule.getSetPosList().isEmpty()) {
+    private void setDayInMonthFromSetPos(final int index, final CalendarObject obj, final Recur rrule) {
+        if (!rrule.getSetPosList().isEmpty()) {
             int firstPos = (Integer) rrule.getSetPosList().get(0);
-            if(firstPos == -1) {
+            if (firstPos == -1) {
                 firstPos = 5;
             }
             obj.setDayInMonth(firstPos);
@@ -332,13 +347,13 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
                     throw new ConversionError(index, "Unknown day: %s", weekday.getDay());
                 }
                 int offset = weekday.getOffset();
-                if(offset != 0) {
-                    if(offset == -1) {
+                if (offset != 0) {
+                    if (offset == -1) {
                         offset = 5;
                     }
                     cObj.setDayInMonth(offset);
                 }
-                
+
                 days |= day.intValue();
             }
             cObj.setDays(days);
@@ -390,4 +405,128 @@ public class Recurrence<T extends CalendarComponent, U extends CalendarObject> e
             cObj.setDays(days);
         }
     }
+    
+    /**
+     * Determines the {@link net.fortuna.ical4j.model.Date} from the supplied
+     * recurring calendar object, ready-to-use in ical4j components. <p/>
+     * While date-only until dates are used as is (for tasks and whole day 
+     * appointments), date-time specific until dates are calculated based on 
+     * the appointments timezone to include the last-possible start-time of 
+     * the last occurrence.    
+     * 
+     * @param calendarObject the recurring calendar object
+     * @return the calculated until date
+     * @see http://tools.ietf.org/html/rfc5545#section-3.3.10
+     */
+    private net.fortuna.ical4j.model.Date getUntil(U calendarObject) {
+        if (calendarObject.containsUntil()) {
+            /*
+             * OX stores series end as date without time (00:00 UTC).
+             */            
+            if (isWholeDay(calendarObject)) {
+                /*
+                 * use DATE value type with UTC time - equal to the stored date
+                 */
+                return new net.fortuna.ical4j.model.Date(calendarObject.getUntil());
+            } 
+            /*
+             * Since DTSTART is specified as date with local time and time zone 
+             * reference, use DATE-TIME value type with UTC time for non-whole-day
+             * events.
+             */
+            Calendar utcUntilCalendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+            utcUntilCalendar.setTime(calendarObject.getUntil());
+            /* 
+             * The effective last possible until date-time is 23:59:59 on the 
+             * series until date in the recurring appointments timezone, so we 
+             * first need to assume the series end being in the appointment's 
+             * timezone, and afterwards add this portion of time. 
+             */
+            java.util.TimeZone appointmentTimeZone = getTimeZone(calendarObject); 
+            Calendar effectiveUntilCalendar = Calendar.getInstance(appointmentTimeZone);
+            effectiveUntilCalendar.set(
+                utcUntilCalendar.get(Calendar.YEAR), 
+                utcUntilCalendar.get(Calendar.MONTH), 
+                utcUntilCalendar.get(Calendar.DAY_OF_MONTH), 
+                23, 59, 59);
+            /*
+             * finally, build an ical4j date-time
+             */
+            net.fortuna.ical4j.model.DateTime dateTime = new net.fortuna.ical4j.model.DateTime(true);
+            dateTime.setTime(effectiveUntilCalendar.getTime().getTime());
+            return dateTime;
+        }
+
+        return null;
+    }
+
+    /**
+     * Extracts the {@link java.util.Date} from the <code>UNTIL</code> 
+     * parameter of the supplied {@link Recur} instance, ready-to-use in 
+     * calendar objects. <p/> 
+     * The value is transformed to an UTC date (without time) implicitly, 
+     * based on the parent component's timezone. Doing so, this date represents 
+     * the date of the last occurrence, compatible with the server's internal 
+     * handling of the "until" property for calendar objects.
+     * 
+     * @param recur the recurrence rule
+     * @param component the parent component
+     * @return the extracted until date
+     * @see http://tools.ietf.org/html/rfc5545#section-3.3.10
+     */
+    private java.util.Date getUntil(Recur recur, T component) {
+        if (null != recur && null != recur.getUntil()) {
+            net.fortuna.ical4j.model.Date until = recur.getUntil();
+            if (net.fortuna.ical4j.model.DateTime.class.isInstance(until)) {
+                /*
+                 * consider DATE-TIME value type with UTC time - determine date 
+                 * of the last occurrence in the appointment's timezone  
+                 */
+                Calendar effectiveUntilCalendar = Calendar.getInstance(getTimeZone(component));
+                effectiveUntilCalendar.setTime(until);
+                /*
+                 * determine OX until date based on the effective until date 
+                 */
+                Calendar utcUntilCalendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+                utcUntilCalendar.set(
+                    effectiveUntilCalendar.get(Calendar.YEAR), 
+                    effectiveUntilCalendar.get(Calendar.MONTH), 
+                    effectiveUntilCalendar.get(Calendar.DAY_OF_MONTH), 
+                    0, 0, 0);
+                utcUntilCalendar.set(Calendar.MILLISECOND, 0);
+                return utcUntilCalendar.getTime();
+            } else {
+                /*
+                 * consider DATE value type - already in OX format
+                 */
+                return until;
+            }
+        }
+        return null;
+    }
+    
+    private boolean isWholeDay(U calendarObject) {
+        return null != calendarObject && (Task.class.isInstance(calendarObject) ||
+            (Appointment.class.isInstance(calendarObject) && ((Appointment)calendarObject).getFullTime()));
+    }
+    
+    private TimeZone getTimeZone(U calendarObject) {
+        String timeZoneID = null;
+        if (null != calendarObject && Appointment.class.isInstance(calendarObject)) {
+            timeZoneID = ((Appointment)calendarObject).getTimezone();
+        }
+        return java.util.TimeZone.getTimeZone(null != timeZoneID ? timeZoneID : "UTC");
+    }
+    
+    private TimeZone getTimeZone(T component) {
+        TimeZone timeZone = null;
+        if (null != component) {
+            DtStart dtStart = (DtStart)component.getProperty(Property.DTSTART);
+            if (null != dtStart) {
+                timeZone = dtStart.getTimeZone();
+            }
+        }
+        return null != timeZone ? timeZone : TimeZone.getTimeZone("UTC");
+    }
+
 }
