@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,30 +53,22 @@ import java.util.List;
 import java.util.TimeZone;
 import net.fortuna.ical4j.model.component.VToDo;
 import net.fortuna.ical4j.model.property.Status;
-import com.openexchange.data.conversion.ical.ConversionError;
 import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.Mode;
 import com.openexchange.data.conversion.ical.ical4j.internal.AbstractVerifyingAttributeConverter;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.tasks.Task;
 
 /**
- *
  * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
  */
 public final class State extends AbstractVerifyingAttributeConverter<VToDo, Task> {
 
-    /**
-     * Default constructor.
-     */
     public State() {
         super();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void emit(final int index, final Task task, final VToDo vtodo,
-                     final List<ConversionWarning> warnings, final Context ctx, Object... args) throws ConversionError {
+    public void emit(final Mode mode, final int index, final Task task, final VToDo vtodo, final List<ConversionWarning> warnings, final Context ctx, final Object... args) {
         try {
             final Status status = new Status(toStatus(index, task.getStatus()).getValue());
             vtodo.getProperties().add(status);
@@ -85,25 +77,15 @@ public final class State extends AbstractVerifyingAttributeConverter<VToDo, Task
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean hasProperty(final VToDo vtodo) {
         return null != vtodo.getStatus();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public boolean isSet(final Task task) {
         return task.containsStatus();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void parse(final int index, final VToDo vtodo, final Task task, final TimeZone timeZone, final Context ctx,
-        final List<ConversionWarning> warnings) throws ConversionError {
+    public void parse(final int index, final VToDo vtodo, final Task task, final TimeZone timeZone, final Context ctx, final List<ConversionWarning> warnings) {
         try {
             task.setStatus(toTask(index, vtodo.getStatus()));
         } catch (final ConversionWarning e) {
@@ -128,8 +110,7 @@ public final class State extends AbstractVerifyingAttributeConverter<VToDo, Task
             retval = Status.VTODO_CANCELLED;
             break;
         default:
-            throw new ConversionWarning(index, ConversionWarning.Code.INVALID_STATUS,
-                Integer.valueOf(taskState));
+            throw new ConversionWarning(index, ConversionWarning.Code.INVALID_STATUS, Integer.valueOf(taskState));
         }
         return retval;
     }
@@ -145,8 +126,7 @@ public final class State extends AbstractVerifyingAttributeConverter<VToDo, Task
         } else if (Status.VTODO_CANCELLED.equals(status)) {
             retval = Task.DEFERRED;
         } else {
-            throw new ConversionWarning(index,ConversionWarning.Code.INVALID_STATUS,
-                status.getValue());
+            throw new ConversionWarning(index, ConversionWarning.Code.INVALID_STATUS, status.getValue());
         }
         return retval;
     }
